@@ -6,8 +6,8 @@ import numpy as np
 from Bio import Phylo
 from Bio.Phylo.BaseTree import Clade
 from Bio import AlignIO
-from treetime import config as ttconf
-from treetime import MissingDataError,UnknownMethodError
+from . import config as ttconf
+from . import MissingDataError,UnknownMethodError
 from .seq_utils import seq2prof, seq2array, prof2seq, normalize_profile, extend_profile
 from .gtr import GTR
 from .gtr_site_specific import GTR_site_specific
@@ -1584,7 +1584,7 @@ class TreeAnc(object):
             return the reconstructed sequence also for terminal nodes. this will replace
             ambiguous sites with the most likely sequence state.
         as_string : bool, optional
-            return the sequence as character array rather than contiguous string
+            return the sequence as contiguous string rather than a character array
         compressed : bool, optional
             return the a sequence where unique alignment patterns are reduced to
             one alignment column each
@@ -1611,8 +1611,10 @@ class TreeAnc(object):
         else:
             if (not reconstructed) and (node.name in self.data.aln):
                 tmp_seq = self.data.aln[node.name]
-            else:
+            elif node.cseq is not None:
                 tmp_seq = self.data.compressed_to_full_sequence(node.cseq, as_string=False)
+            else:
+                tmp_seq = np.array([self.gtr.ambiguous or 'N']*self.sequence_length)
 
         return "".join(tmp_seq) if as_string else np.copy(tmp_seq)
 
